/* TRHelp -- V1.0
   Thomas Regner
   jan 1997
   class-methods of TRHelpBuilder
   */
#include <fstream.h>
#include <string.h>
#include "TRHelpBuilder.h"
#include <ctype.h>

TRHelpBuilder::TRHelpBuilder()
{
    verbose = FALSE;
}

boolean TRHelpBuilder::BuildTargetFile(ifstream &input)
{
    if(verbose)
        cout << "start BuildTarget.." << endl;
    char c;
    char buffer[10];
    int buffcount = 0;
    int lineNumber = 1;
    ofstream temploc;
    temploc.open("temploc");
    ofstream tempout;
    tempout.open("tempout");
    boolean newline = FALSE;
    
    while(input.get(c))
    {
        switch(c)
        {    
            case escapeMark:
                    // consume the following char if it is not
                    // a linkMark
                input.get(c);
                switch(c)
                {
                    case EOF:
                        tempout << c;
                        exit;
                        break;
                    case linkMark:
                            // a linkMark has to be escaped until
                            // links can be handled
                        tempout << "\\" << c;
                        break;
                    default:
                        tempout << c;
                        break;
                }
                break;
            case tagMark:
                    // read tag-text and write it with li.no. in temp-file
                input.get(c);
                while( ! isspace(c) && c != '\n')
                {
                    buffer[buffcount] = c;
                    buffcount++;
                    input.get(c);
                }
                if( c == '\n')
                {  // memory a read newline for later care
                    tempout << "\n";
                    newline = TRUE;
                }
                    // never forget the final StringEnd
                buffer[buffcount] = '\0';
                temploc << buffer << endl << lineNumber << endl;
                if(verbose)
                    cout << "Memory tag : " << buffer << " at line "
                         << lineNumber << "." << endl;
                buffcount = 0;
                if(newline)
                {  // take care of read newline
                    lineNumber++;
                    newline = FALSE;
                }
                break;
            case commentMark:
                    // consume comments and forget them 
                input.get(c);
                while( c != '\n')
                    input.get(c);
                break;
            case '\n':
                    // lineNumber actualisation 
                lineNumber++;
                tempout << "\n";
                break;
            default:
                    // just put it out in tempfile
                tempout << c;
                break;
        } 
    }
    if( verbose)
        cout << "End BuildTarget..." << endl;
    return TRUE;
}

boolean TRHelpBuilder::BuildLocFile(ofstream &output, ofstream &location)
{
    if(verbose)
        cout << endl << "Start BuildLoc..." << endl;
    strcpy (ErrorText, "OK\0");
    char c;
    char ibuffer[10];
    char obuffer[10];
    int ibuffcount = 0;
    int lineNumber = 1;
    int targetLineNumber;
    int refcount = 0;
    
    ifstream temploc;
    temploc.open("temploc");
    
    ifstream tempin;
    tempin.open("tempout");
        // this cost's space (at least up to TRH_MAX_TAG*sizeof(int)
        // byte 
    location << "int TRHlinks["<< TRHMaxRef << "][2]={";
    boolean ready = FALSE;
    boolean newline = FALSE;
    
    while( tempin.get(c))
    {
        switch(c)
        {
            case escapeMark:
                tempin.get(c);
                switch(c)
                {
                    case linkMark:
                        output << c;
                        exit;
                        break;
                    default:
                        output << "\\" << c;
                        break;
                }
                break;
            case linkMark:
                    // get link-text and try to find tag lineNumber
                tempin.get(c);
                while(! isspace(c) && c != '\n')
                {
                    ibuffer[ibuffcount] = c;
                    ibuffcount++;
                    tempin.get(c);
                }
                ibuffer[ibuffcount] = '\0';
                if( c == '\n')
                {   // memory newline for later care
                    output << "\n";
                    newline = TRUE;
                }
                
                while( ! ready)
                {   // try to find corresponding tag text
                    if( readline( obuffer, temploc) == EOF)
                    {
                        strcpy( ErrorText, "Untaged references\0");
                        return FALSE;
                    }        
                    if( strcmp(obuffer, ibuffer) == 0)
                        ready = TRUE;
                }
                    // next line contains tag line-number
                targetLineNumber = readint( temploc);
                    // put it out to header file
                location << endl << "{ " << lineNumber << ", " <<
                    targetLineNumber << "},";
                refcount++;
                if( refcount > (TRHMaxRef - 1))
                {
                    strcpy( ErrorText, "RefCount exceeded\0");
                    return FALSE;
                }
                
                if(verbose)
                    cout << " link from line " << lineNumber
                         << " to line " << targetLineNumber << "." <<
                        endl;
                ibuffcount = 0;
                ready = FALSE;
                if(newline)
                {   // take care of newline
                    lineNumber++;
                    newline = FALSE;
                }
                    // to rewind tag-mark file
                    // there must be a better way...
                    // what to do under C++ ?
                temploc.close();
                temploc.open("temploc");
                break;
            case '\n':
                lineNumber++;
                output << "\n";
                break;
            default:
                output << c;
                break;
        }
    }
    location << "\n{-1,-1}};\n";
    return TRUE;
}

char TRHelpBuilder::readline( char* string, ifstream &file)
{
    int pos = 0;
    char c;
    file.get(c);
    while( ( c != '\n') && ( pos <= ( tagSize )))
    {
            // do not read a number-line
        if( isdigit(c))
        {
            file.get(c);
            while( c != '\n')
                file.get(c);
            file.get(c);
        }
            // EOF during string-read 
        if( c == EOF)
            exit;
            // everythings ok
        string[pos] = c;
        pos++;
        file.get(c);
    }
    string[pos] = '\0';
    return c;
}

int TRHelpBuilder::readint( ifstream &file)
{
    char buffer[tagSize + 1];
    char c;
    int pos = 0;
    
    file.get(c);
    while( c != '\n' && pos <= tagSize - 1)
    {
        if( ( ! isdigit(c)) || ( c == EOF))
            return -1;
        buffer[pos] = c;
        pos++;
        file.get(c);
    }
    buffer[pos] = '\0';
    return Str2Int( buffer);
}

int TRHelpBuilder::Str2Int( const char * string)
{
    int result = 0;
    int decimal = 1;
    int figure;
    int pos = 0;
    
    while( isdigit(string[pos]) )
        pos++;
    pos--;
    while( pos >= 0)
    {
        figure = string[pos] - '0';
        result += figure * decimal;
        pos--;
        decimal *= 10;
    }
    return result;
}

void TRHelpBuilder::SetVerboseFlag( boolean v)
{
    verbose = v;
}

void TRHelpBuilder::GetErrorText( char* text)
{
    strcpy( text, ErrorText);
}

